* Data synthesis Chapter
drop _all

clear
matrix P = (1,.2\.2,1)
matrix A1 = cholesky(P)
matrix list A1
set obs 40
set seed 13215
generate xo1 = rnormal()
generate xo2 = rnormal()

generate x1 = xo1
generate x2 = A1[2,1]*xo1 + A1[2,2]*xo2
correlate x1 x2

correlate xo1 xo2, covariance
matrix A2 = cholesky(syminv(r(C)))
generate z1 = xo1*A2[1,1]+xo2*A2[2,1]
generate z2 = (xo1*A2[1,1]+xo2*A2[2,1])*A1[2,1] + ///
     (xo1*A2[1,2]+xo2*A2[2,2])*A1[2,2]
correlate z1 z2

drop _all
set obs 20
matrix exch = J(3,3,.4) + .6*I(3)
corr2data err1 err2 err3, corr(exch)
correlate err*

generate panel = _n
reshape long err, i(panel) j(t)
quietly xtset panel
xtgee err
estat wcorrelation


drop _all
set seed 2394
drawbinmodel 50000 3, corr(exch 0.25)
xtset id
xtgee y a b, family(binomial) link(logit) nolog
estat wcorrelation


drop _all
set seed 2393
set obs 10000
generate x1  = mod((_n-1),4)==1 | mod((_n-1),4)==3
generate x2  = mod((_n-1),4)==2 | mod((_n-1),4)==3
generate eta = 1.00+1.00*x1+1.00*x2  // Linear predictor with specified beta
generate mu  = eta                   // mu as the inverse link function of eta

generate yp = rnormal(mu)

generate ys = rnormal(mu)
forvalues x1=0/1 {
    forvalues x2=0/1 {
       quietly summarize ys if x1==`x1' & x2==`x2'
       quietly replace ys = (ys-r(mean))/r(sd)+eta if x1==`x1' & x2==`x2'
    }
}

table x1 x2, contents(mean yp sd yp)
table x1 x2, contents(mean ys sd ys)

glm yp x1 x2, noheader
glm ys x1 x2, noheader

drop _all
set seed 2393
set obs 50
generate x1  = mod((_n-1),4)==1 | mod((_n-1),4)==3
generate x2  = mod((_n-1),4)==2 | mod((_n-1),4)==3
generate eta = 1.00+1.00*x1+1.00*x2  // Linear predictor with specified beta
generate mu  = eta                   // mu as the inverse link function of eta
generate yp  = rnormal(mu)
generate ys  = rnormal(mu)
forvalues x1=0/1 {
    forvalues x2=0/1 {
        quietly summarize ys if x1==`x1' & x2==`x2'
        quietly replace ys = (ys-r(mean))/r(sd) + eta if x1==`x1' & x2==`x2'
    }
}
glm yp x1 x2, noheader
glm ys x1 x2, noheader

drop _all
set seed 8721
set obs 10000
generate x1  = mod((_n-1),2)==1 
generate eta = 1.00+1.00*x1  // Linear predictor with specified beta
generate mu  = invlogit(eta) // mu as the inverse link function of eta

generate yp = runiform() < mu

generate ys   = .
generate flag = 0
forvalues x1=0/1 {
    quietly {
        replace flag = -(x1==`x1')
        count if flag==-1
        local N = r(N)
        sort flag
        replace ys = _n < round(mu*`N',1) in 1/`N'
    }
}

glm yp x1, family(binomial) link(logit) noheader nolog
glm ys x1, family(binomial) link(logit) noheader nolog

table ys x1, contents(freq)

drop _all
set seed 8721
set obs 50
generate x1  = mod((_n-1),2)==1 
generate eta = 1.00+1.00*x1  // Linear predictor with specified beta
generate mu  = invlogit(eta) // mu as the inverse link function of eta
generate yp = runiform() < mu
generate ys   = .
generate flag = 0
forvalues x1=0/1 {
    quietly {
        replace flag = -(x1==`x1')
        count if flag==-1
        local N = r(N)
        sort flag
        replace ys = _n < round(mu*`N',1) in 1/`N'
    }
}
glm yp x1, family(binomial) link(logit) noheader nolog
glm ys x1, family(binomial) link(logit) noheader nolog

drop _all
set seed 23224
set obs 50000
generate x1  = mod((_n-1),2)==1 
generate eta = 1.00-1.50*x1  // Linear predictor with specified beta
generate mu  = normal(eta)   // mu as the inverse link function of eta
generate yp  = runiform() < mu
glm yp x1, family(binomial) link(probit) noheader nolog

capture program drop Mysim
program Mysim, rclass
    replace x1 = rnormal()
    replace eta= 1.00-1.50*x1
    replace mu = normal(eta)
    replace yp = runiform() < mu
    glm yp x1, family(binomial) link(logit) iterate(10)
    return scalar logit = e(aic)
    glm yp x1, family(binomial) link(probit) iterate(10)
    return scalar probit = e(aic)
end
drop _all
set seed 23224
set obs 500
foreach var in x1 eta mu yp {
    quietly generate `var' = .
}
simulate probit=r(probit) logit=r(logit), reps(100): Mysim
count if probit < logit
di "Probit is preferred " r(N) " times out of 100"

drop _all
set seed 82344
set obs 50000
generate x1  = mod((_n-1),2)==1 
generate eta = 0.50-0.50*x1     // Linear predictor with specified beta
generate mu  = 1-exp(-exp(eta)) // mu as the inverse link function of eta
generate yp  = runiform() < mu
glm yp x1, family(binomial) link(cloglog) noheader nolog

generate ys = .
generate flag = 0
forvalues x1=0/1 {
    quietly {
        replace flag = -(x1==`x1')
        count if flag==-1
        local N = r(N)
        sort flag
        replace ys = _n < round(mu*`N',1) in 1/`N'
    }
}
glm ys x1, family(binomial) link(cloglog) noheader nolog


drop _all
set seed 2393
set obs 10000
generate x1  = mod((_n-1),4)==1 | mod((_n-1),4)==3
generate x2  = mod((_n-1),4)==2 | mod((_n-1),4)==3
generate eta = 1.00+1.00*x1+0.00*x2  // Linear predictor with specified beta
generate mu  = exp(eta)              // mu as the inverse link function of eta
generate y   = rnormal(mu, 2.0)
generate ly  = log(y)

glm y  x1 x2, family(gaussian) link(log) noheader nolog
glm ly x1 x2, family(gaussian) link(identity) noheader nolog

capture program drop GPquantile
program GPquantile, rclass
    args u mu k
    quietly {
        local mu = exp(`mu')
        local maxval = 10000
        local x = 0
        local p = exp(ln((1-`k')*`mu') + (`x'-1)*ln((1-`k')*`mu'+`k'*`x') ///
            - (1-`k')*`mu' - `k'*`x' - lngamma(`x'+1))
        while `p' <`u' & `x' < `maxval' {
            local x = `x'+1
            local p = `p'+ exp(ln((1-`k')*`mu') + (`x'-1)*ln((1-`k')*`mu'+ ///
                `k'*`x') - (1-`k')*`mu' - `k'*`x' - lngamma(`x'+1))
        }
        return scalar x = `x'
    }
end

capture program drop rgpoisson
program rgpoisson 
    args ry xb delta
    quietly {
        tempvar u
        generate `u' = runiform()
        capture drop `ry'
        generate `ry' = .
        describe, short
        local N = r(N)
        forvalues i=1/`N' {
            GPquantile (`u'[`i']) (`xb'[`i']) (`delta')
            replace `ry' = r(x) in `i'
        }
    }
end

drop _all
set seed 29843
set obs 40
generate x1 = runiform() < .5
generate x2 = runiform() < .5
generate eta = 1 + 0.25*x1 - 0.25*x2
rgpoisson y eta (-0.3)

gpoisson y x1 x2, nolog
glm y x1 x2, family(poisson) nolog


set seed 28373
capture program drop Mysim
program Mysim, rclass
    syntax [, noCONStant]
    quietly {
        if "`constant'" == "" {
            local scale = 1
        }
        else {
            tempvar scale
            generate `scale' = 2*runiform()
        }
        replace x1  = runiform()
        replace x2  = runiform()
        replace eta = 0.00+1.00*x1+2.00*x2
        replace mu  = eta
        replace y = rnormal(mu)*sqrt(`scale')
        capture glm y x1 x2, family(gaussian) link(identity)
        return scalar z = (_b[x1]-1)/_se[x1]
    }
end

capture drop _all
set obs 30
foreach var in x1 x2 eta mu y {
    quietly generate `var' = .
}
simulate z = r(z), reps(1000) nodots: Mysim 
label var z "Test statistic"
histogram z, kdensity plot(function stdnorm = normalden(x,0,1),    ///
    ra(-3 3) lpattern(dash)) title(Distribution of test statistic) ///
    subtitle(Homoskedastic Variance)
graph export reghomo.eps, replace


set seed 23984
capture drop _all
set obs 30
foreach var in x1 x2 eta mu y {
    quietly generate `var' = .
}
generate scale = 2*runiform()
simulate z = r(z), reps(1000) nodots: Mysim, noconstant
label var z "Test statistic"
histogram z, kdensity plot(function stdnorm = normalden(x,0,1),    ///
    ra(-3 3) lpattern(dash)) title(Distribution of test statistic) ///
    subtitle(Heteroskedastic Variance)
graph export reghetero.eps, replace



set seed 87232
capture program drop Mysim
program Mysim, rclass
    args delta
    quietly {
        replace x1  = rnormal(0)
        replace eta = 0.00+(1.00+`delta')*x1
        replace mu  = eta
        replace y = rnormal(mu)
        capture glm y x1, family(gaussian) link(identity)
        return scalar z = (_b[x1]-1)/_se[x1]
    }
end

capture program drop Mypow
program Mypow
    syntax, Vals(numlist min=1 >0 <1 sort) ///
        [Nobs(integer 30) Reps(integer 1000) ALPha(real .05)]
    quietly {
        noisily display
        foreach delta in `vals' {
            capture drop _all
            set obs `nobs'
            foreach var in x1 eta mu y {
                generate `var' = .
            }
            simulate z = r(z), reps(`reps') nodots: Mysim `delta'
            count if abs(z) > invnormal(1-`alpha'/2) 
            local p = r(N)/`reps'
            noi display as txt "P(delta=" as res %4.2f `delta' ///
                as txt ") = " as res %6.4f `p'
        }
    }
end
Mypow, vals(0.2 0.5 0.8)


capture program drop Mysim
program Mysim, rclass
    quietly {
        replace x1 = rnormal()
        replace x2 = rnormal()
        replace nby = rpoisson(rgamma(1/.5,.10)*exp(0.75*x1-1.25*x2+2))
        capture glm nby x1 x2, family(nbinomial ml) iterate(10)
        return scalar AICnb = e(aic)
        capture glm nby x1 x2, family(poisson) iterate(10)
        return scalar AICpo = e(aic)
    }
end

drop _all
set obs  30
set seed 88530
quietly generate nby = .
quietly generate x1  = .
quietly generate x2  = .
simulate negbin=r(AICnb) poisson=r(AICpo), reps(100): Mysim
count if negbin < poisson


capture program drop Mysim
program Mysim, rclass
    quietly {
        replace x1 = rnormal()
        replace x2 = rnormal()
        replace nby = rpoisson(rgamma(1/.5,.10)*exp(0.75*x1-1.25*x2+2))
        capture glm nby x1 x2, family(poisson) iterate(10)
        return scalar z = (_b[x1]-0.75)/_se[x1]
    }
end

drop _all
set obs  30
set seed 45323
quietly generate nby = .
quietly generate x1  = .
quietly generate x2  = .
simulate z=r(z), reps(1000) nodots: Mysim
label var z "Test statistic assuming Poisson variance"
histogram z, kdensity plot(function stdnorm = normalden(x,0,1),    ///
    ra(-3 3) lpattern(dash)) title(Distribution of test statistic) ///
    subtitle(Poisson Variance)
graph export poissvar.eps, replace


capture program drop Mysim
program Mysim, rclass
    quietly {
        replace x1 = rnormal()
        replace x2 = rnormal()
        replace nby = rpoisson(rgamma(1/.5,.10)*exp(0.75*x1-1.25*x2+2))
        capture glm nby x1 x2, family(poisson) iterate(10) scale(x2)
        return scalar z = (_b[x1]-0.75)/_se[x1]
    }
end

drop _all
set obs  30
set seed 23553
quietly generate nby = .
quietly generate x1  = .
quietly generate x2  = .
simulate z=r(z), reps(1000) nodots: Mysim
label var z "Test statistic assuming quasi-Poisson variance"
histogram z, kdensity plot(function stdnorm = normalden(x,0,1),    ///
    ra(-3 3) lpattern(dash)) title(Distribution of test statistic) ///
    subtitle(Quasi-Poisson Variance)
graph export qpoissvar.eps, replace

capture program drop Mysim
program Mysim, rclass
    syntax [, n(integer 30) a(real 0.0)]
    quietly {
        drop _all
        set obs `n'
        generate x1 = runiform() < .5
        generate x2 = runiform() < .5
        generate x3 = rnormal()
        generate y  = rpoisson(exp(x1+2*x2+`a'*x3))
        summarize y
        local den = r(Var)*(r(N)-1)
        glm y x1 x2, family(poisson) iterate(10)
        predict yhat
        generate num = (y-yhat)^2
        summarize num
        return scalar r2=1-r(sum)/`den'
    }
end

set seed 4982
forvalues a=0(10)100 {
    quietly {
        local alpha = `a'/100
        simulate r2=r(r2), reps(100) nodots: Mysim, n(30) a(`alpha')
        summarize r2
        noi display "Efron Pseudo-R2[a=" %3.2f (`alpha') "]=" %6.4f r(mean)
    }
}

